@@ -815,8 +815,8 @@ class ZipFile implements ZipConstants, Closeable {
                 public boolean startsWithLocHeader(ZipFile zip) {
                     return zip.startsWithLocHeader();
                 }
-                public int getManifestNum(JarFile zip) {
-                    return ((ZipFile)zip).getManifestNum();
+                public int getManifestNum(JarFile jar) {
+                    return ((ZipFile)jar).getManifestNum();
                 }
              }
         );
@@ -865,4 +865,13 @@ class ZipFile implements ZipConstants, Closeable {
     private static native byte[] getEntryBytes(long jzentry, int type);
 
     private static native String getZipMessage(long jzfile);
+
+    private static native byte[] getCentralDirectoryBytes(long jzfile) throws ZipException;
+
+    private byte[] getCentralDirectory() throws ZipException {
+        synchronized (this) {
+            ensureOpen();
+            return getCentralDirectoryBytes(jzfile);
+        }
+    }
 }
